@@ -2,15 +2,20 @@ import datetime
 import functools
 import inspect
 import logging
+import math
+import os
 import random
+import subprocess
 import sys
 import uuid
 from enum import Enum
+from pathlib import Path
 from typing import Any, Dict, Iterable, List, Optional, Tuple, Union, no_type_check
 from warnings import warn
 
 import colorama
 import pandas as pd
+import requests
 from typeguard import typechecked
 
 from gantry.config import Config
@@ -22,7 +27,11 @@ from gantry.exceptions import (
 from gantry.logger.stores import APILogStore, BaseLogStore
 from gantry.query.core.utils import check_response
 from gantry.serializers import serializable_value
-from gantry.utils import compute_feedback_id, format_msg_with_color, check_event_time_in_future
+from gantry.utils import (
+    check_event_time_in_future,
+    compute_feedback_id,
+    format_msg_with_color,
+)
 from gantry.validators import validate_logged_field_datatype
 
 logger = logging.getLogger(__name__)
@@ -50,6 +59,10 @@ def _batch_success_msg(batch_id, application, log_store: Union[APILogStore, Base
         logger.info("Look for batch id: {}".format(batch_id))
 
 
+def get_file_linecount(filepath):
+    return int(subprocess.check_output(["wc", "-l", filepath]).split()[0])
+
+
 def _build_prediction_and_feedback_events(
     application: str,
     context: Dict[str, str],
@@ -323,10 +336,24 @@ def _log_exception(func):
     return wrapper
 
 
+def _is_empty(data: Union[List[Any], pd.DataFrame, Any] = None) -> bool:
+    """
+    Check whether input data is empty or not
+    Returns True if data is None or empty, False if data size >= 1
+    """
+    if isinstance(data, pd.DataFrame):
+        return data.empty
+    return True if not data else False
+
+
+def _is_not_empty(data: Union[List[Any], pd.DataFrame, Any] = None) -> bool:
+    return not _is_empty(data)
+
+
 class Gantry:
     def __init__(
         self,
-        log_store: Union[BaseLogStore, APILogStore],
+        log_store: APILogStore,
         environment: str,
         logging_level: str = "INFO",
     ):
@@ -464,6 +491,89 @@ class Gantry:
 
         return outer
 
+    @_log_exception
+    @typechecked
+    def log_file(
+        self,
+        application: str,
+        filepath: str,
+        version: Optional[Union[int, str]] = None,
+        timestamp: Optional[str] = None,
+        inputs: List[str] = [],
+        outputs: List[str] = [],
+        feedbacks: List[str] = [],
+        metadata: List[str] = [],
+    ):
+        # Read the headers of the file, infer and build linking datastruct.
+        gantry_data_link: Dict[str, Any] = {
+            "file_type": "CSV_WITH_HEADERS",
+            "timestamp": dict(),
+            "inputs": dict(),
+            "outputs": dict(),
+            "feedback": dict(),
+            "metadata": dict(),
+        }
+        with open(filepath, "r") as f_in:
+            headers_line = f_in.readline().split(",")
+        for col_index, raw_header in enumerate(headers_line):
+            header = raw_header.strip()
+            if header == timestamp or (not timestamp and header.startswith("timestamp")):
+                gantry_data_link["timestamp"] = {"ref": col_index}
+            elif header in outputs or header.startswith("output"):
+                gantry_data_link["outputs"][header] = {"ref": col_index}
+            elif header in feedbacks or header.startswith("feedback"):
+                gantry_data_link["feedback"][header] = {"ref": col_index}
+            elif header in metadata or header.startswith("metadata"):
+                gantry_data_link["metadata"][header] = {"ref": col_index}
+            else:
+                gantry_data_link["inputs"][header] = {"ref": col_index}
+
+        # Get info for tracking batch during processing.
+        line_count = get_file_linecount(filepath)
+        logger.info(f"Processing {line_count - 1} lines.")
+
+        # Calculate part counts and generate presigned s3 urls.
+        path = Path(filepath)
+        CHUNK_SIZE = 10 * 1024 * 1024  # Chunk size 10 MB
+        object_size = os.path.getsize(filepath)  # in bytes
+        total_num_parts = math.ceil(object_size / CHUNK_SIZE)
+
+        preupload_res = self.log_store._api_client.request(
+            "GET",
+            "/api/v1/ingest/file-preupload",
+            params={"filename": path.name, "num_parts": total_num_parts},
+        )
+        signed_urls = preupload_res["upload_urls"]
+        upload_id = preupload_res["upload_id"]
+        file_key = preupload_res["key"]
+
+        # Upload the parts.
+        parts = []
+        with open(filepath, "r") as f_in:
+            block_read = functools.partial(f_in.read, CHUNK_SIZE)
+            block_iterator = iter(block_read, "")
+            for part_num, block in enumerate(block_iterator):
+                # TODO: parallelize this client side?
+                response = requests.put(signed_urls[part_num], data=block)
+                etag = str(response.headers["ETag"])
+                parts.append({"ETag": etag, "PartNumber": part_num + 1})
+
+        # Submit file completion to gantry API.
+        logger.info("Initializing Upload to Gantry")
+        self.log_store._api_client.request(
+            "POST",
+            "/api/v1/ingest/file",
+            json={
+                "upload_id": upload_id,
+                "key": file_key,
+                "parts": parts,
+                "data_link": gantry_data_link,
+            },
+            headers={"Content-Type": "application/json", "Accept": "application/json"},
+        )
+        # batch_id = complete_res["batch_id"]
+        # _batch_success_msg(batch_id, application, self.log_store)
+
     @_log_exception
     @typechecked
     def log_records(
@@ -546,23 +656,23 @@ class Gantry:
                 in the 'batch' section of the dashboard
 
         """
-        preds_exist = (inputs is not None) and (outputs is not None)
-        feedbacks_exist = feedbacks is not None
+        preds_exist = _is_not_empty(inputs) and _is_not_empty(outputs)
+        feedbacks_exist = _is_not_empty(feedbacks)
 
         if preds_exist and feedbacks_exist:
             return self._log_predictions_and_feedbacks(
-                application,
-                version,
-                inputs,
-                outputs,
-                feedback_keys,
-                feedbacks,
-                ignore_inputs,
-                timestamps,
-                reference_name,
-                sort_on_timestamp,
-                sample_rate,
-                as_batch,
+                application=application,
+                inputs=inputs,
+                outputs=outputs,
+                feedbacks=feedbacks,
+                version=version,
+                feedback_keys=feedback_keys,
+                ignore_inputs=ignore_inputs,
+                timestamps=timestamps,
+                reference_name=reference_name,
+                sort_on_timestamp=sort_on_timestamp,
+                sample_rate=sample_rate,
+                as_batch=as_batch,
             )
         elif preds_exist:
             return self.log_predictions(
@@ -605,11 +715,11 @@ class Gantry:
     def _log_predictions_and_feedbacks(
         self,
         application: str,
+        inputs: Union[List[dict], pd.DataFrame],
+        outputs: Union[List[dict], pd.DataFrame],
+        feedbacks: Union[List[dict], pd.DataFrame],
         version: Optional[Union[int, str, List[Union[str, int]]]] = None,
-        inputs: Union[List[dict], pd.DataFrame] = None,
-        outputs: Union[List[dict], List[Any], pd.DataFrame] = None,
         feedback_keys: Optional[List[str]] = None,
-        feedbacks: Union[List[dict], pd.DataFrame] = None,
         ignore_inputs: Optional[List[str]] = None,
         timestamps: Optional[List[datetime.datetime]] = None,
         reference_name: Optional[str] = None,
@@ -618,7 +728,7 @@ class Gantry:
         as_batch: bool = False,
     ) -> None:
         """Internal method to log batch of predictions AND feedbacks"""
-        if not (inputs and outputs and feedbacks):
+        if any(_is_empty(d) for d in [inputs, outputs, feedbacks]):
             raise GantryLoggingException(
                 "In order to use log_records method, inputs/outputs/feedbacks need to be provided"
             )
@@ -761,8 +871,8 @@ class Gantry:
             sample_rate (optional, float): Specify the probability as a float that the event will
                 be sent to Gantry.
         """
-        pred_exist = (inputs is not None) and (outputs is not None)
-        feedback_exist = feedback is not None
+        pred_exist = _is_not_empty(inputs) and _is_not_empty(outputs)
+        feedback_exist = _is_not_empty(feedback)
 
         if pred_exist and feedback_exist:
             return self._log_prediction_and_feedback_event(
@@ -804,10 +914,10 @@ class Gantry:
     def _log_prediction_and_feedback_event(
         self,
         application: str,
+        inputs: dict,
+        outputs: Any,
+        feedback: Optional[dict],
         version: Optional[Union[int, str]] = None,
-        inputs: dict = None,
-        outputs: Any = None,
-        feedback: Optional[dict] = None,
         feedback_keys: Optional[List[str]] = None,
         ignore_inputs: Optional[List[str]] = None,
         timestamp: Optional[datetime.datetime] = None,
