@@ -96,10 +96,10 @@ class APILogStore(BaseLogStore):
         Pings the API to check if it is up and running. Returns True if alive, else False.
         """
         try:
-            response = self._api_client.request(
-                "GET",
-                "/healthz/readiness",
-            )
+            # Cannot use /healthz/* endpoints as those will be answered by nginx
+            # need to use /.
+            # See https://linear.app/gantry/issue/ENG-2978/revisit-ping-in-sdk
+            response = self._api_client.request("GET", "/api/ping")
             check_response(response)
             return True
         except Exception as e:
