@@ -10,7 +10,6 @@ import pytest
 import responses
 from freezegun import freeze_time
 
-from gantry import Gantry
 from gantry.config import Config
 from gantry.exceptions import GantryBatchCreationException, GantryLoggingDataTypeError
 from gantry.logger import client
@@ -335,7 +334,7 @@ def test_log_feedback_event(mock_uuid, timestamp, cli_obj):
 @mock.patch("gantry.logger.client.uuid.uuid4")
 def test_log_feedback_event_sample_rate_0_5(mock_uuid, timestamp):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.4):
         gantry.log_feedback_event(
@@ -354,7 +353,7 @@ def test_log_feedback_event_sample_rate_0_5(mock_uuid, timestamp):
 @mock.patch("gantry.logger.client.uuid.uuid4")
 def test_log_feedback_event_sample_rate_0_5_none(mock_uuid, timestamp):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
     with mock.patch("random.random", return_value=0.6):
         mock_uuid.return_value = "12345"
         gantry.log_feedback_event(
@@ -373,7 +372,7 @@ def test_log_feedback_event_sample_rate_0_5_none(mock_uuid, timestamp):
 @mock.patch("gantry.logger.client.uuid.uuid4")
 def test_log_feedback_event_sample_rate_0(mock_uuid, timestamp):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.4):
         mock_uuid.return_value = "12345"
@@ -566,7 +565,7 @@ def test_log_prediction_event(mock_uuid, feedback_keys, feedback_id, timestamp,
 @mock.patch("gantry.logger.client.uuid.uuid4")
 def test_log_prediction_event_sample_rate_0_5(mock_uuid, feedback_keys, feedback_id, timestamp):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.4):
         mock_uuid.return_value = "12345"
@@ -598,7 +597,7 @@ def test_log_prediction_event_sample_rate_0_5_none(
     mock_uuid, feedback_keys, feedback_id, timestamp
 ):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.6):
         mock_uuid.return_value = "12345"
@@ -628,7 +627,7 @@ def test_log_prediction_event_sample_rate_0_5_none(
 @mock.patch("gantry.logger.client.uuid.uuid4")
 def test_log_prediction_event_sample_rate_0(mock_uuid, feedback_keys, feedback_id, timestamp):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.6):
         mock_uuid.return_value = "12345"
@@ -868,7 +867,7 @@ def test_log_prediction_and_feedback_event(mock_uuid, timestamp, cli_obj):
 @mock.patch("gantry.logger.client.uuid.uuid4")
 def test_log_prediction_and_feedback_event_sample_rate_0_5(mock_uuid, timestamp):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.4):
         mock_uuid.return_value = "12345"
@@ -889,7 +888,7 @@ def test_log_prediction_and_feedback_event_sample_rate_0_5(mock_uuid, timestamp)
 @mock.patch("gantry.logger.client.uuid.uuid4")
 def test_log_prediction_and_feedback_event_sample_rate_0_5_none(mock_uuid, timestamp):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.6):
         mock_uuid.return_value = "12345"
@@ -910,7 +909,7 @@ def test_log_prediction_and_feedback_event_sample_rate_0_5_none(mock_uuid, times
 @mock.patch("gantry.logger.client.uuid.uuid4")
 def test_log_prediction_and_feedback_event_sample_rate_0(mock_uuid, timestamp):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.6):
         mock_uuid.return_value = "12345"
@@ -1080,7 +1079,7 @@ def test_log_feedback(mock_uuid, feedbacks, random_error, timestamps, sort, cli_
 @mock.patch("gantry.logger.client.uuid.uuid4")
 def test_log_feedback_sample_rate_0_5(mock_uuid, feedbacks, timestamps, sort):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.4):
         mock_uuid.return_value = "ABCD1234"
@@ -1110,7 +1109,7 @@ def test_log_feedback_sample_rate_0_5(mock_uuid, feedbacks, timestamps, sort):
 @mock.patch("gantry.logger.client.uuid.uuid4")
 def test_log_feedback_sample_rate_0_5_none(mock_uuid, feedbacks, timestamps, sort, cli_obj):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.6):
         mock_uuid.return_value = "ABCD1234"
@@ -1140,7 +1139,7 @@ def test_log_feedback_sample_rate_0_5_none(mock_uuid, feedbacks, timestamps, sor
 @mock.patch("gantry.logger.client.uuid.uuid4")
 def test_log_feedback_sample_rate_0_0(mock_uuid, feedbacks, timestamps, sort):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.6):
         mock_uuid.return_value = "ABCD1234"
@@ -1443,7 +1442,7 @@ def test_log_predictions_sample_rate_0_5(
     mock_uuid, inputs, outputs, timestamps, reference_name, sort
 ):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.4):
         mock_uuid.return_value = "ABCD1234"
@@ -1485,7 +1484,7 @@ def test_log_predictions_sample_rate_0_5_none(
     mock_uuid, inputs, outputs, timestamps, reference_name, sort
 ):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.6):
         mock_uuid.return_value = "ABCD1234"
@@ -1527,7 +1526,7 @@ def test_log_predictions_sample_rate_0(
     mock_uuid, inputs, outputs, timestamps, reference_name, sort
 ):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.1):
         mock_uuid.return_value = "ABCD1234"
@@ -1814,20 +1813,51 @@ def test_log_predictions_and_feedbacks_size_mismatch_versions(version, cli_obj,
 @pytest.mark.parametrize("timestamps", [None, [ANOTHER_TIME, SOME_TIME]])
 @pytest.mark.parametrize("sort", [False, True])
 @pytest.mark.parametrize("as_batch", [False, True])
+@pytest.mark.parametrize(
+    "test_feedbacks",
+    [
+        [{"A": 200}, {"A": 201}],
+        pd.DataFrame.from_dict({"A": [200, 201]}),
+    ],
+)
+@pytest.mark.parametrize(
+    "test_outputs",
+    [
+        [{"B": 300}, {"B": 301}],
+        pd.DataFrame.from_dict({"B": [300, 301]}),
+    ],
+)
+@pytest.mark.parametrize(
+    "test_inputs",
+    [
+        [{"A": 100}, {"A": 101}],
+        pd.DataFrame.from_dict({"A": [100, 101]}),
+    ],
+)
 @mock.patch("gantry.logger.client.uuid.uuid4")
 @mock.patch("gantry.logger.client.Gantry._create_batch")
-def test_log_predictions_and_feedbacks(
-    mock_create_batch, mock_uuid, as_batch, sort, timestamps, reference_name, cli_obj, log_store
+def test_log_predictions_and_feedbacks_df(
+    mock_create_batch,
+    mock_uuid,
+    test_inputs,
+    test_outputs,
+    test_feedbacks,
+    as_batch,
+    sort,
+    timestamps,
+    reference_name,
+    cli_obj,
+    log_store,
 ):
     mock_create_batch.return_value = {"id": "batch-id-123"}
     mock_uuid.side_effect = ["ABCD1", "ABCD2", "ABCD3", "ABCD4"]
     cli_obj._log_predictions_and_feedbacks(
         application="foobar",
         version="2.0.1",
-        inputs=[{"A": 100}, {"A": 101}],
-        outputs=[{"B": 300}, {"B": 301}],
+        inputs=test_inputs,
+        outputs=test_outputs,
         feedback_keys=["A"],
-        feedbacks=[{"A": 200}, {"A": 201}],
+        feedbacks=test_feedbacks,
         timestamps=timestamps,
         reference_name=reference_name,
         sort_on_timestamp=sort,
@@ -1894,19 +1924,46 @@ def test_log_predictions_and_feedbacks(
         mock_create_batch.assert_not_called()
 
 
+@pytest.mark.parametrize(
+    "test_feedbacks",
+    [
+        [{"A": 200}, {"A": 201}],
+        pd.DataFrame.from_dict({"A": [200, 201]}),
+    ],
+)
+@pytest.mark.parametrize(
+    "test_outputs",
+    [
+        [{"B": 300}, {"B": 301}],
+        pd.DataFrame.from_dict({"B": [300, 301]}),
+    ],
+)
+@pytest.mark.parametrize(
+    "test_inputs",
+    [
+        [{"A": 100}, {"A": 101}],
+        pd.DataFrame.from_dict({"A": [100, 101]}),
+    ],
+)
 @mock.patch("gantry.logger.client.Gantry._log_predictions_and_feedbacks")
 @mock.patch("gantry.logger.client.Gantry.log_predictions")
 @mock.patch("gantry.logger.client.Gantry.log_feedback")
 def test_log_records_preds_and_feedback(
-    mock_feedback, mock_preds, mock_preds_and_feedback, cli_obj
+    mock_feedback,
+    mock_preds,
+    mock_preds_and_feedback,
+    test_inputs,
+    test_outputs,
+    test_feedbacks,
+    cli_obj,
 ):
     cli_obj.log_records(
         application="foobar",
         version="2.0.1",
-        inputs=[{"A": 100}, {"A": 101}],
-        outputs=[{"B": 300}, {"B": 301}],
+        inputs=test_inputs,
+        outputs=test_outputs,
         feedback_keys=["A"],
-        feedbacks=[{"A": 200}, {"A": 201}],
+        feedbacks=test_feedbacks,
         timestamps=None,
         reference_name="some-tag",
         sort_on_timestamp=False,
@@ -1917,18 +1974,39 @@ def test_log_records_preds_and_feedback(
     mock_feedback.assert_not_called()
 
 
-def test_log_records_preds_and_feedback_0_5():
+@pytest.mark.parametrize(
+    "test_feedbacks",
+    [
+        [{"A": 200}, {"A": 201}],
+        pd.DataFrame.from_dict({"A": [200, 201]}),
+    ],
+)
+@pytest.mark.parametrize(
+    "test_outputs",
+    [
+        [{"B": 300}, {"B": 301}],
+        pd.DataFrame.from_dict({"B": [300, 301]}),
+    ],
+)
+@pytest.mark.parametrize(
+    "test_inputs",
+    [
+        [{"A": 100}, {"A": 101}],
+        pd.DataFrame.from_dict({"A": [100, 101]}),
+    ],
+)
+def test_log_records_preds_and_feedback_0_5(test_inputs, test_outputs, test_feedbacks):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.4):
         gantry.log_records(
             application="foobar",
             version="2.0.1",
-            inputs=[{"A": 100}, {"A": 101}],
-            outputs=[{"B": 300}, {"B": 301}],
+            inputs=test_inputs,
+            outputs=test_outputs,
             feedback_keys=["A"],
-            feedbacks=[{"A": 200}, {"A": 201}],
+            feedbacks=test_feedbacks,
             timestamps=None,
             reference_name="some-tag",
             sort_on_timestamp=False,
@@ -1939,18 +2017,39 @@ def test_log_records_preds_and_feedback_0_5():
     log_store.log_batch.assert_called_once()
 
 
-def test_log_records_preds_and_feedback_0_5_none():
+@pytest.mark.parametrize(
+    "test_feedbacks",
+    [
+        [{"A": 200}, {"A": 201}],
+        pd.DataFrame.from_dict({"A": [200, 201]}),
+    ],
+)
+@pytest.mark.parametrize(
+    "test_outputs",
+    [
+        [{"B": 300}, {"B": 301}],
+        pd.DataFrame.from_dict({"B": [300, 301]}),
+    ],
+)
+@pytest.mark.parametrize(
+    "test_inputs",
+    [
+        [{"A": 100}, {"A": 101}],
+        pd.DataFrame.from_dict({"A": [100, 101]}),
+    ],
+)
+def test_log_records_preds_and_feedback_0_5_none(test_inputs, test_outputs, test_feedbacks):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.6):
         gantry.log_records(
             application="foobar",
             version="2.0.1",
-            inputs=[{"A": 100}, {"A": 101}],
-            outputs=[{"B": 300}, {"B": 301}],
+            inputs=test_inputs,
+            outputs=test_outputs,
             feedback_keys=["A"],
-            feedbacks=[{"A": 200}, {"A": 201}],
+            feedbacks=test_feedbacks,
             timestamps=None,
             reference_name="some-tag",
             sort_on_timestamp=False,
@@ -1961,18 +2060,39 @@ def test_log_records_preds_and_feedback_0_5_none():
     log_store.log_batch.assert_not_called()
 
 
-def test_log_records_preds_and_feedback_0_0():
+@pytest.mark.parametrize(
+    "test_feedbacks",
+    [
+        [{"A": 200}, {"A": 201}],
+        pd.DataFrame.from_dict({"A": [200, 201]}),
+    ],
+)
+@pytest.mark.parametrize(
+    "test_outputs",
+    [
+        [{"B": 300}, {"B": 301}],
+        pd.DataFrame.from_dict({"B": [300, 301]}),
+    ],
+)
+@pytest.mark.parametrize(
+    "test_inputs",
+    [
+        [{"A": 100}, {"A": 101}],
+        pd.DataFrame.from_dict({"A": [100, 101]}),
+    ],
+)
+def test_log_records_preds_and_feedback_0_0(test_inputs, test_outputs, test_feedbacks):
     log_store = mock.MagicMock()
-    gantry = Gantry(log_store=log_store, environment="test")
+    gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.6):
         gantry.log_records(
             application="foobar",
             version="2.0.1",
-            inputs=[{"A": 100}, {"A": 101}],
-            outputs=[{"B": 300}, {"B": 301}],
+            inputs=test_inputs,
+            outputs=test_outputs,
             feedback_keys=["A"],
-            feedbacks=[{"A": 200}, {"A": 201}],
+            feedbacks=test_feedbacks,
             timestamps=None,
             reference_name="some-tag",
             sort_on_timestamp=False,
@@ -1983,15 +2103,31 @@ def test_log_records_preds_and_feedback_0_0():
     log_store.log_batch.assert_not_called()
 
 
+@pytest.mark.parametrize(
+    "test_outputs",
+    [
+        [{"B": 300}, {"B": 301}],
+        pd.DataFrame.from_dict({"B": [300, 301]}),
+    ],
+)
+@pytest.mark.parametrize(
+    "test_inputs",
+    [
+        [{"A": 100}, {"A": 101}],
+        pd.DataFrame.from_dict({"A": [100, 101]}),
+    ],
+)
 @mock.patch("gantry.logger.client.Gantry._log_predictions_and_feedbacks")
 @mock.patch("gantry.logger.client.Gantry.log_predictions")
 @mock.patch("gantry.logger.client.Gantry.log_feedback")
-def test_log_records_preds_only(mock_feedback, mock_preds, mock_preds_and_feedback, cli_obj):
+def test_log_records_preds_only(
+    mock_feedback, mock_preds, mock_preds_and_feedback, test_inputs, test_outputs, cli_obj
+):
     cli_obj.log_records(
         application="foobar",
         version="2.0.1",
-        inputs=[{"A": 100}, {"A": 101}],
-        outputs=[{"B": 300}, {"B": 301}],
+        inputs=test_inputs,
+        outputs=test_outputs,
         feedback_keys=["A"],
         feedbacks=None,
         timestamps=None,
@@ -2004,17 +2140,26 @@ def test_log_records_preds_only(mock_feedback, mock_preds, mock_preds_and_feedba
     mock_feedback.assert_not_called()
 
 
+@pytest.mark.parametrize(
+    "test_feedbacks",
+    [
+        [{"A": 200}, {"A": 201}],
+        pd.DataFrame.from_dict({"A": [200, 201]}),
+    ],
+)
 @mock.patch("gantry.logger.client.Gantry._log_predictions_and_feedbacks")
 @mock.patch("gantry.logger.client.Gantry.log_predictions")
 @mock.patch("gantry.logger.client.Gantry.log_feedback")
-def test_log_records_feedback_only(mock_feedback, mock_preds, mock_preds_and_feedback, cli_obj):
+def test_log_records_feedback_only(
+    mock_feedback, mock_preds, mock_preds_and_feedback, test_feedbacks, cli_obj
+):
     cli_obj.log_records(
         application="foobar",
         version="2.0.1",
         inputs=None,
         outputs=None,
         feedback_keys=["A"],
-        feedbacks=[{"A": 200}, {"A": 201}],
+        feedbacks=test_feedbacks,
         timestamps=None,
         reference_name="some-tag",
         sort_on_timestamp=False,
@@ -2025,17 +2170,37 @@ def test_log_records_feedback_only(mock_feedback, mock_preds, mock_preds_and_fee
     mock_feedback.assert_called_once()
 
 
+@pytest.mark.parametrize(
+    "test_feedbacks",
+    [None, pd.DataFrame(), []],
+)
+@pytest.mark.parametrize(
+    "test_outputs",
+    [None, pd.DataFrame(), []],
+)
+@pytest.mark.parametrize(
+    "test_inputs",
+    [None, pd.DataFrame(), []],
+)
 @mock.patch("gantry.logger.client.Gantry._log_predictions_and_feedbacks")
 @mock.patch("gantry.logger.client.Gantry.log_predictions")
 @mock.patch("gantry.logger.client.Gantry.log_feedback")
-def test_log_records_no_data(mock_feedback, mock_preds, mock_preds_and_feedback, cli_obj):
+def test_log_records_no_data(
+    mock_feedback,
+    mock_preds,
+    mock_preds_and_feedback,
+    test_inputs,
+    test_outputs,
+    test_feedbacks,
+    cli_obj,
+):
     cli_obj.log_records(
         application="foobar",
         version="2.0.1",
-        inputs=None,
-        outputs=None,
+        inputs=test_inputs,
+        outputs=test_outputs,
         feedback_keys=["A"],
-        feedbacks=None,
+        feedbacks=test_feedbacks,
         timestamps=None,
         reference_name="some-tag",
         sort_on_timestamp=False,
@@ -2075,17 +2240,23 @@ def test_log_record_pred_and_feedback(mock_feedback, mock_pred, mock_pred_and_fe
     mock_feedback.assert_not_called()
 
 
+@pytest.mark.parametrize(
+    "test_feedback",
+    [None, {}],
+)
 @mock.patch("gantry.logger.client.Gantry._log_prediction_and_feedback_event")
 @mock.patch("gantry.logger.client.Gantry.log_prediction_event")
 @mock.patch("gantry.logger.client.Gantry.log_feedback_event")
-def test_log_record_pred_only(mock_feedback, mock_pred, mock_pred_and_feedback, cli_obj):
+def test_log_record_pred_only(
+    mock_feedback, mock_pred, mock_pred_and_feedback, test_feedback, cli_obj
+):
     cli_obj.log_record(
         application="foobar",
         version="2.0.1",
         inputs={"A": 100},
         outputs={"B": 300},
         feedback_keys=["A"],
-        feedback=None,
+        feedback=test_feedback,
         timestamp=None,
     )
 
@@ -2104,15 +2275,25 @@ def test_log_record_pred_only(mock_feedback, mock_pred, mock_pred_and_feedback,
     mock_feedback.assert_not_called()
 
 
+@pytest.mark.parametrize(
+    "test_output",
+    [None, {}],
+)
+@pytest.mark.parametrize(
+    "test_input",
+    [None, {}],
+)
 @mock.patch("gantry.logger.client.Gantry._log_prediction_and_feedback_event")
 @mock.patch("gantry.logger.client.Gantry.log_prediction_event")
 @mock.patch("gantry.logger.client.Gantry.log_feedback_event")
-def test_log_record_feedback_only(mock_feedback, mock_pred, mock_pred_and_feedback, cli_obj):
+def test_log_record_feedback_only(
+    mock_feedback, mock_pred, mock_pred_and_feedback, test_input, test_output, cli_obj
+):
     cli_obj.log_record(
         application="foobar",
         version="2.0.1",
-        inputs=None,
-        outputs=None,
+        inputs=test_input,
+        outputs=test_output,
         feedback_keys=["A"],
         feedback={"A": 200},
         timestamp=None,
@@ -2130,14 +2311,18 @@ def test_log_record_feedback_only(mock_feedback, mock_pred, mock_pred_and_feedba
     )
 
 
+@pytest.mark.parametrize(
+    "test_input",
+    [None, {}],
+)
 @mock.patch("gantry.logger.client.Gantry._log_prediction_and_feedback_event")
 @mock.patch("gantry.logger.client.Gantry.log_prediction_event")
 @mock.patch("gantry.logger.client.Gantry.log_feedback_event")
-def test_log_record_no_data(mock_feedback, mock_pred, mock_pred_and_feedback, cli_obj):
+def test_log_record_no_data(mock_feedback, mock_pred, mock_pred_and_feedback, test_input, cli_obj):
     cli_obj.log_record(
         application="foobar",
         version="2.0.1",
-        inputs=None,
+        inputs=test_input,
         outputs={"B": 100},
         feedback_keys=["A"],
         feedback=None,
